/*
 * *****************************************************************************************
 *
 * 		Copyright (C) 2014-2021 Gowin Semiconductor Technology Co.,Ltd.
 * 		
 * @file        main.c
 * @author      Embedded Development Team
 * @version     V1.0.1
 * @date        2021-04-29 09:00:00
 * @brief       Main function.
 ******************************************************************************************
 */

/* Includes ------------------------------------------------------------------*/
#include "uart.h"
#include "multiple.h"

/* Declarations ------------------------------------------------------------------*/
static char* int2str(int num,char*str,int radix);
static char buff[20];

/* Functions ------------------------------------------------------------------*/
int main()
{
  SystemInit();  //Initializes system clock
  UartInit();    //Initializes UART0
	
  UART_SendString(UART0, "\r\n*********************************************\r\n");
  UART_SendString(UART0, "                   AHB Arbiter                 \r\n");
  UART_SendString(UART0, "*********************************************\r\n");
	
  UART_SendString(UART0, "\r\nSystem initialization finished.\r\n");
  UART_SendString(UART0, "UART0 initialization finished.\r\n");
  UART_SendString(UART0, "Multiple initialization finished.\r\n");
  UART_SendString(UART0, "Initialization Status : \r\n");

  //First multiplier
  UART_SendString(UART0, "\r\n----- Start first multiple -----\r\n");
  setMultiplier(20);
  setMultiplicand(40);
  startMultiple();
  while(getFinishStatus()==FINISHED_STATUS);
  finishMultiple();
  UART_SendString(UART0, "Multiplier = ");
  UART_SendString(UART0, int2str(getMultiplier(), buff, 10));
  UART_SendString(UART0, "\r\n");
  UART_SendString(UART0, "Multiplicand = ");
  UART_SendString(UART0, int2str(getMultiplicand(), buff, 10));
  UART_SendString(UART0, "\r\n");
  UART_SendString(UART0, "Result = ");
  UART_SendString(UART0, int2str(getMultipleResult(), buff, 10));
  UART_SendString(UART0, "\r\n");
  UART_SendString(UART0, "Multiple first finished.\r\n");

  //Second multiplier
  UART_SendString(UART0, "\r\n----- Start second multiple -----\r\n");
  setMultiplier(30);
  setMultiplicand(50);
  startMultiple();
  while(getFinishStatus()==FINISHED_STATUS);
  finishMultiple();
  UART_SendString(UART0, "Multiplier = ");
  UART_SendString(UART0, int2str(getMultiplier(), buff, 10));
  UART_SendString(UART0, "\r\n");
  UART_SendString(UART0, "Multiplicand = ");
  UART_SendString(UART0, int2str(getMultiplicand(), buff, 10));
  UART_SendString(UART0, "\r\n");
  UART_SendString(UART0, "Result = ");
  UART_SendString(UART0, int2str(getMultipleResult(), buff, 10));
  UART_SendString(UART0, "\r\n");
  UART_SendString(UART0, "Multiple second finished.\r\n");

  while(1);
}

//integer to string
static char* int2str(int num,char*str,int radix)
{
	char index[]="0123456789ABCDEF";
	unsigned unum;
	int i=0,j,k;
	char temp;

	if(radix==10&&num<0)
	{
		unum=(unsigned)-num;
		str[i++]='-';
	}
	else
	{
		unum=(unsigned)num;
	}

	do
	{
		str[i++]=index[unum%(unsigned)radix];
		unum/=radix;
	}while(unum);
	str[i]='\0';

	if(str[0]=='-')
	{
		k=1;
	}
	else
	{
		k=0;
	}

	for(j=k;j<=(i-1)/2;j++)
	{
		temp=str[j];
		str[j]=str[i-1+k-j];
		str[i-1+k-j]=temp;
	}

	return str;
}
